import numpy as np
import torch

from dataclasses import dataclass
import requests


@dataclass
class Trainer:
    model: any = None
    data_manager: any = None
    ip: str = "0.0.0.0"
    port: int = 5000
    epochs: int = 1

    def __post_init__(self):
        self.train_loss = list()
        self.val_loss = list()

    def train(self):
        train_loader = self.data_manager.train_loader
        val_loader = self.data_manager.validation_loader

        for epoch in range(self.epochs):
            loss = []
            for batch_idx, data_target in enumerate(train_loader):
                data = data_target[0]
                target = data_target[1]
                loss.append(self.model.step(data, target))
            epoch_loss = np.mean(loss)
            self.train_loss.append(epoch_loss)
            self.send_watcher('loss', 'training', epoch_loss, len(self.train_loss))
            with torch.no_grad():
                loss = []
                for data_target in val_loader:
                    data = data_target[0]
                    target = data_target[1]
                    loss.append(self.model.loss(data, target))
            epoch_loss = np.mean(loss)
            self.val_loss.append(epoch_loss)
            self.send_watcher('loss', 'validation', epoch_loss, len(self.val_loss))

        return {
            'train_loss': self.train_loss,
            'validation_loss': self.val_loss,
        }

    def send_watcher(self, metric, mode, value, idx):
        server = f'http://{self.ip}:{self.port}/'
        endpoint = ""
        if metric == 'loss':
            endpoint = "updateLoss"

        body = {
            'data': {
                'name': self.model.name,
                'mode': mode,
                'value': float(value),
                'index': idx
            }
        }

        url = server + endpoint
        requests.post(url, json=body)



import json
import os
import socket
from datetime import datetime

import requests
import torch
import torch.nn as nn
import torch.nn.functional as F
import torch.optim as optim


class Net(nn.Module):
    def __init__(self):
        super().__init__()
        self.conv1 = nn.Conv2d(1, 5, (2, 100))
        self.pool = nn.MaxPool2d(2, 2)
        self.conv2 = nn.Conv2d(5, 10, (2, 100))
        self.fc1 = nn.Linear(1194240, 120)
        self.fc2 = nn.Linear(120, 84)
        self.fc3 = nn.Linear(84, 2)

    def forward(self, x):
        x = self.pool(F.relu(self.conv1(x)))
        x = self.pool(F.relu(self.conv2(x)))
        x = torch.flatten(x, 1)  # flatten all dimensions except batch
        x = F.relu(self.fc1(x))
        x = F.relu(self.fc2(x))
        x = self.fc3(x)
        return x


class Model:
    def __init__(self, net=Net(), name="Model", save_dir="./results/", device=None, lr=0.001):
        self.creation_date = str(datetime.now())
        self.net = net
        self.criterion = nn.CrossEntropyLoss()
        self.optimizer = optim.Adam(self.net.parameters(), lr=lr)
        if device is None:
            self.device = torch.device('cuda:0' if torch.cuda.is_available() else 'cpu')
        else:
            self.device = device
        self.net.to(device=self.device)
        self.name = name
        self.save_dir = os.path.join(save_dir, name)
        self.save_name = os.path.join(self.save_dir, f"{self.name}.pth")
        self.lr = lr
        self.verify_save_dir(os.path.join(save_dir, name))

    def step(self, inputs, labels):
        inputs = inputs.to(self.device)
        labels = labels.to(self.device)
        self.optimizer.zero_grad()
        outputs = self.net(inputs)
        loss = self.criterion(outputs, labels)
        loss.backward()
        self.optimizer.step()
        return loss.item()

    def loss(self, inputs, labels):
        inputs = inputs.to(self.device)
        labels = labels.to(self.device)
        outputs = self.net(inputs)
        loss = self.criterion(outputs, labels)
        return loss.item()

    def classify(self, inputs):
        return self.net(inputs.to(self.device))

    def save(self, mode=None, ip=None, port=None):
        if mode is not None:
            save_name = os.path.join(self.save_dir, f"{self.name}-{mode}.pth")
            torch.save(self.net.state_dict(), save_name)
        else:
            torch.save(self.net.state_dict(), self.save_name)

        if ip is not None and port is not None:
            res = requests.get(f'http://{ip}:{port}/download')
            json_dict = json.loads(res.text)
            if not os.path.isdir(self.save_dir):
                os.mkdir(self.save_dir)
            with open(os.path.join(self.save_dir, f"{self.name}-{mode}-performance-metrics.json"), 'w') as file_out:
                json.dump(json_dict[self.name], file_out)

    def load(self):
        self.net.load_state_dict(torch.load(self.save_name))

    def verify_save_dir(self, directory):
        if not os.path.exists(directory):
            os.makedirs(directory)

    def get_model_info(self):
        return_dict = dict()
        for attribute, value in vars(self).items():
            return_dict[attribute] = str(value)

        return_dict['host'] = socket.getfqdn()

        return return_dict



from collections import deque
from dataclasses import dataclass
from http.client import RemoteDisconnected
import json
import os
import requests
import threading

import numpy as np

from ml_infrastructure.app import start
from ml_infrastructure.trainer import Trainer
from ml_infrastructure.evaluator import Evaluator


@dataclass
class Manager:
    models: any = None
    data_manager: any = None
    ip: str = "0.0.0.0"
    port: int = 5000
    epochs: int = 100
    start_watcher_app: bool = True
    window_size: int = 20
    eval_rate: int = 1

    def __post_init__(self):
        if self.models is not None:
            self.trainers = [Trainer(model, self.data_manager, self.ip, self.port, self.epochs) for model in
                             self.models]
            self.evaluators = [Evaluator(model, self.data_manager, self.ip, self.port) for model in self.models]

        if self.start_watcher_app:
            self.start_watcher()

    def start_watcher(self):
        threading.Thread(target=start, args=(self.ip, int(self.port),)).start()

    def register_model(self, model):
        post_body = {
            'data': model.get_model_info()
        }
        url = f'http://{self.ip}:{self.port}/registerModel'

        requests.post(url=url, json=post_body)

    def perform(self):

        for trainer, evaluator in zip(self.trainers, self.evaluators):
            epoch_num = 0
            while True:
                epoch_num += 1
                loss = trainer.train()

                if min(loss['validation_loss']) == loss['validation_loss'][-1]:
                    trainer.model.save(mode="Lowest-Val-Loss", ip=self.ip, port=self.port)

                if epoch_num%self.eval_rate != 0:
                    continue

                performance = evaluator.evaluate()
                stopping_criteria = requests.get(f"http://{self.ip}:{self.port}/stopCriteria")
                stopping_criteria = json.loads(stopping_criteria.text)

                if stopping_criteria['threshold'] != -1 or stopping_criteria['epoch'] != -1:
                    if stopping_criteria['threshold'] != -1:
                        performance = performance['validation'][stopping_criteria['metric']][stopping_criteria['index']]
                        if performance >= stopping_criteria['threshold']:
                            trainer.model.save(mode="Final", ip=self.ip, port=self.port)
                            break
                    elif stopping_criteria['epoch'] != -1:
                        if epoch_num >= int(stopping_criteria['epoch']):
                            trainer.model.save(mode="Final", ip=self.ip, port=self.port)
                            break
                else:
                    validation_loss = deque(maxlen=self.window_size)
                    [validation_loss.append(l) for l in loss['validation_loss']]
                    if len(validation_loss) == self.window_size:
                        slope = np.polyfit(range(0, len(validation_loss)), validation_loss, 1)[0]
                        if -0.001 < slope:
                            trainer.model.save(mode="Final", ip=self.ip, port=self.port)
                            break

    def shutdown_watcher(self):
        try:
            requests.get(f'http://{self.ip}:{self.port}/shutdown')
        except RemoteDisconnected as e:
            print(f"Shutdown Watcher at {self.ip}:{self.port}")

    def get_watcher_results(self):
        res = requests.get(f'http://{self.ip}:{self.port}/download')
        return res.text

    def save_watcher_results(self, save_location, save_name):
        text = self.get_watcher_results()
        if not os.path.isdir(save_location):
            os.mkdir(save_location)

        with open(os.path.join(save_location, save_name), 'w') as file_out:
            json.dump(text, file_out)


if __name__ == "__main__":
    manager = Manager()

    manager.start_watcher()



import torch.nn as nn
import torch
import torch.nn.functional as F
import torchvision
import torchvision.transforms as transforms

from ml_infrastructure.model import Model
from ml_infrastructure.data_manager import DataManager
from ml_infrastructure.manager import Manager

if __name__ == "__main__":
    # Create a Network
    class LeNet(nn.Module):
        def __init__(self):
            super().__init__()
            self.conv1 = nn.Conv2d(3, 6, 5)
            self.pool = nn.MaxPool2d(2, 2)
            self.conv2 = nn.Conv2d(6, 16, 5)
            self.fc1 = nn.Linear(16 * 5 * 5, 120)
            self.fc2 = nn.Linear(120, 84)
            self.fc3 = nn.Linear(84, 10)

        def forward(self, x):
            x = F.max_pool2d(F.relu(self.conv1(x)), (2, 2))
            # If the size is a square, you can specify with a single number
            x = F.max_pool2d(F.relu(self.conv2(x)), 2)
            x = torch.flatten(x, 1)  # flatten all dimensions except the batch dimension
            x = F.relu(self.fc1(x))
            x = F.relu(self.fc2(x))
            x = self.fc3(x)
            return x

    class SomeCNN(nn.Module):
        def __init__(self):
            super().__init__()
            self.conv1 = nn.Conv2d(3, 32, 2)
            self.pool = nn.MaxPool2d(2, 2)
            self.conv2 = nn.Conv2d(32, 64, 2)
            self.conv3 = nn.Conv2d(64, 128, 2)
            self.fc1 = nn.Linear(128 * 3 * 3, 288)
            self.fc2 = nn.Linear(288, 288)
            self.fc3 = nn.Linear(288, 128)
            self.fc4 = nn.Linear(128, 10)

        def forward(self, x):
            x = self.pool(F.relu(self.conv1(x)))
            x = self.pool(F.relu(self.conv2(x)))
            x = self.pool(F.relu(self.conv3(x)))
            x = torch.flatten(x, 1)  # flatten all dimensions except batch
            x = F.relu(self.fc1(x))
            x = F.relu(self.fc2(x))
            x = F.relu(self.fc3(x))
            x = self.fc4(x)
            return x


    myLenet = Model(net=LeNet(), name="Example_Lenet", save_dir='./results', device=None, lr=0.001)
    myCNN = Model(net=SomeCNN(), name="Example_CNN", save_dir='./results', device=None, lr=0.001)

    # Create The Data Loader
    transform = transforms.Compose([transforms.ToTensor(),
                                    transforms.Normalize((0.5, 0.5, 0.5), (0.5, 0.5, 0.5))])

    location = './data'
    batch_size = 50

    train_set = torchvision.datasets.CIFAR10(root=location, train=True,
                                             download=True, transform=transform)
    train_loader = torch.utils.data.DataLoader(train_set, batch_size=batch_size,
                                               shuffle=True, num_workers=2)

    test_set = torchvision.datasets.CIFAR10(root=location, train=False,
                                            download=True, transform=transform)
    test_loader = torch.utils.data.DataLoader(test_set, batch_size=batch_size,
                                              shuffle=False, num_workers=2)

    classes = ('plane', 'car', 'bird', 'cat',
     'deer', 'dog', 'frog', 'horse', 'ship', 'truck')

    dm = DataManager(train_loader, test_loader, test_loader, classes)

    # Create The Manager

    manager = Manager(models=[myLenet, myCNN], data_manager=dm, epochs=10)

    print("Starting Training and Evaluation")
    manager.perform()

    print("Saving Results")
    manager.save_watcher_results(save_location='./results', save_name='Example.json')

    print("Shutting Down")
    manager.shutdown_watcher()



import datetime
import numpy as np
import torch
import requests
from dataclasses import dataclass


def calc_performance(matrix, model, classes, mode):
    performance = {
        "name": model.name,
        "time": str(datetime.datetime.now()),
        "mode": mode,
        "Accuracy": [],
        "Classification Error": [],
        "TP": [],
        "FP": [],
        "TN": [],
        "FN": [],
        "Precision": [],
        "Recall": [],
        "Specificity": [],
        "F1-Score": [],
        "confusion_matrix": matrix.tolist(),
        "Classes": classes
    }

    for idx in range(len(classes)):
        TP = int(matrix[idx, idx])
        FN = int(np.sum(matrix[:, idx]) - TP)
        FP = int(np.sum(matrix[idx, :]) - TP)
        TN = int(np.sum(matrix) - (FN + FP + TP))
        performance["TP"].append(TP)
        performance["FN"].append(FN)
        performance["FP"].append(FP)
        performance["TN"].append(TN)
        if (TP + FP) != 0:
            performance['Precision'].append(TP / (TP + FP))
        else:
            performance['Precision'].append(0)

        if (TP + FN) != 0:
            performance['Recall'].append(TP / (TP + FN))
        else:
            performance['Recall'].append(0)

        if (TN + FP) != 0:
            performance['Specificity'].append(TN / (TN + FP))
        else:
            performance['Specificity'].append(0)

        if (TP + FP + TN + FN) != 0:
            performance['Accuracy'].append((TP + TN) / (TP + FP + TN + FN))
            performance['Classification Error'].append((FP + FN) / (TP + FP + TN + FN))
        else:
            performance['Accuracy'].append(0)
            performance['Classification Error'].append(0)

        if (performance['Precision'][idx] + performance['Recall'][idx]) != 0:
            performance['F1-Score'].append(((2 * performance['Precision'][idx] * performance['Recall'][idx]) /
                                            (performance['Precision'][idx] + performance['Recall'][idx])))
        else:
            performance['F1-Score'].append(0)

    performance['Total Accuracy'] = ((np.sum(performance['TP']) + np.sum(performance['TN'])) /
                                     (np.sum(performance['TP'])
                                      + np.sum(performance['TN'])
                                      + np.sum(performance['FP'])
                                      + np.sum(performance['FN'])))

    return performance


def send_performance(performance_dict, ip, port):
    server = f'http://{ip}:{port}/'
    endpoint = "/evalUpdate"

    body = {
        'data': performance_dict
    }

    url = server + endpoint
    requests.post(url, json=body)


def evaluate_model(model, data_manager, mode):
    classes = data_manager.classes

    if mode == 'training':
        loader = data_manager.train_loader
    else:
        loader = data_manager.validation_loader

    confusion_matrix = np.zeros((len(classes), len(classes)), dtype=int)

    if len(data_manager.classes) == 2:
        with torch.no_grad():
            for data in loader:
                inputs, targets = data[0], data[1]

                outputs = model.classify(inputs)
                predicted = torch.sigmoid(outputs)
                predicted = predicted.to("cpu")
                predicted = [1 if x >= .5 else 0 for x in predicted]
                for pred, true in zip(predicted, targets):
                    confusion_matrix[int(pred), int(true)] += 1

    return confusion_matrix


@dataclass
class Evaluator:
    model: any = None
    data_manager: any = None
    ip: str = "0.0.0.0"
    port: int = 5000
    best_f1: int = 0
    save_dir: str = "./results/"

    def evaluate(self):
        total_performance = {'training': None, 'validation': None}
        for mode in ['training', 'validation']:
            matrix = evaluate_model(self.model, self.data_manager, mode)
            performance = calc_performance(matrix, self.model, self.data_manager.classes, mode)
            send_performance(performance, self.ip, self.port)

            total_performance[mode] = performance

        self.model.save(mode="current")

        if np.mean(total_performance['validation']['F1-Score']) > self.best_f1:
            self.model.save(mode='best-f1-score', ip=self.ip, port=self.port)
            self.best_f1 = np.mean(total_performance['validation']['F1-Score'])

        return total_performance



from setuptools import setup

VERSION = '1.5.1'
DESCRIPTION = 'Software infrastructure to for machine learning'
LONG_DESCRIPTION = 'Software infrastructure for machine learning projects that makes it easier to manage experiments and log progress'

setup(
    name="ml_infrastructure",
    version=VERSION,
    description=DESCRIPTION,
    long_description_content_type="text/markdown",
    long_description=LONG_DESCRIPTION,
    author="Ada L",
    author_email="the.nostra.tymus@gmail.com",
    license='MIT',
    packages=['ml_infrastructure'],
    install_requires=[
        'flask',
        'turbo_flask',
        'plotly',
        'requests',
        'torch',
        'numpy'
    ],
    package_data={'ml_infrastructure':['templates/*']},
    keywords='machine learning',
    classifiers= [
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        'License :: OSI Approved :: MIT License',
        "Programming Language :: Python :: 3",
    ]
)



import json
import plotly
import tempfile

from flask import Flask, render_template, request, Response, send_file
from turbo_flask import Turbo
from plotly.graph_objs import Figure, Table, Scatter
import numpy as np
import os

app = Flask(__name__)
turbo = Turbo(app)

performance = dict()
idx = 0
stop_criteria = {
    "threshold": -1,
    "epoch": -1
}


@app.route('/')
@app.route('/loss')
def loss():
    return render_template('loss.html')


@app.route('/evaluation')
def evaluation():
    global idx
    idx = int(request.args['index'])
    print(idx)
    return render_template('eval.html')


@app.route('/controller')
def control():
    return render_template('control.html')


@app.route('/console')
def console():
    return render_template('console.html')


@app.route('/stop')
def stopForm():
    return render_template('stopForm.html')


@app.route('/stopCriteria')
def getStopCriteria():
    global stop_criteria

    return Response(json.dumps(stop_criteria), status=200, mimetype='application/json')


@app.route('/updateCriteria')
def updateCriteria():
    global stop_criteria

    if request.args['index'] == "":
        index = 0
    else:
        index = request.args['index']

    if request.args['threshold'] == "":
        threshold = -1
    else:
        threshold = request.args['threshold']

    if request.args['epoch'] == "":
        epoch = -1
    else:
        epoch = request.args['epoch']

    stop_criteria = {
        'metric': request.args['metric'],
        'threshold': threshold,
        'index': index,
        'epoch': epoch
    }

    return Response("Okay", status=200, mimetype='application/json')


@app.route('/updateLoss', methods=["POST"])
def updateLoss():
    global performance
    name = request.json['data']['name']
    mode = request.json['data']['mode']
    if name not in performance.keys():
        performance = add_model(performance, name)

    performance[name][mode]['values'].append(request.json['data']['value'])
    performance[name][mode]['index'].append(request.json['data']['index'])

    if mode == "training":
        turbo.push(turbo.update(render_template('trainingLoss.html'), 'trainingLoss'))
    else:
        turbo.push(turbo.replace(render_template('validationLoss.html'), 'validationLoss'))

    return Response("Okay", status=200, mimetype='application/json')


@app.route('/evalUpdate', methods=["POST"])
def evalUpdate():
    global performance
    evaluation_results = request.json['data']
    name = evaluation_results['name']
    mode = evaluation_results['mode']

    if name not in performance.keys():
        performance = add_model(performance, name)
    performance[name]['evaluation'][mode] = evaluation_results

    if mode == 'training':
        turbo.push(turbo.replace(render_template('trainingEval.html'), 'trainingEval'))
    else:
        turbo.push(turbo.replace(render_template('validationEval.html'), 'validationEval'))

    return Response("Okay", status=200, mimetype='application/json')


@app.route('/save', methods=['GET'])
def save():
    global performance
    save_location = request.args.get('save_location')
    save_name = request.args.get('save_name')
    full_name = os.path.join(save_location, save_name)
    with open(full_name, 'w') as file_out:
        json.dump(performance, file_out)

    return Response(f"Saved: {full_name}", status=200, mimetype='application/json')


@app.route('/restore', methods=['POST'])
def restore():
    global performance
    performance = request.json['data']
    turbo.push(turbo.replace(render_template('trainingEval.html'), 'trainingEval'))
    turbo.push(turbo.replace(render_template('validationEval.html'), 'validationEval'))
    turbo.push(turbo.update(render_template('trainingLoss.html'), 'trainingLoss'))
    turbo.push(turbo.replace(render_template('validationLoss.html'), 'validationLoss'))
    return Response(f"Restored", status=200, mimetype='application/json')


@app.route('/registerModel', methods=['POST'])
def register_model():
    global performance
    model_data = request.json['data']
    name = model_data['name']
    if name not in performance.keys():
        performance = add_model(performance, name)
    performance[name]['model_details'] = model_data

    return Response(f"Registered {name}", status=200, mimetype='application/json')


@app.route('/reset', methods=['GET'])
def reset():
    global performance
    performance = dict()
    return Response(f"Reset Success!", status=200, mimetype='application/json')


@app.route('/download', methods=['GET'])
def download():
    global performance
    temp = tempfile.NamedTemporaryFile()
    with open(temp.name, 'w') as file_out:
        json.dump(performance, file_out)
    return send_file(temp.name, download_name="Evaluation_Metrics.json")


@app.route('/shutdown', methods=['GET'])
def shutdown():
    os._exit(0)


@app.context_processor
def inject_load():
    return {'trainingLossJSON': get_loss_graph('training'), 'validationLossJSON': get_loss_graph('validation'),
            'trainingLogLossJSON': get_loss_graph_log('training'),
            'validationLogLossJSON': get_loss_graph_log('validation'), 'trainingEvalJSON': get_eval_table('training'),
            'validationEvalJSON': get_eval_table('validation')}


def get_loss_graph(mode):
    global performance
    loss_graph = {
        'data': [Scatter(x=performance[key][mode]['index'],
                         y=performance[key][mode]['values'],
                         name=key) for key in performance.keys()],
        'layout': {
            'title': f'<b> {mode.capitalize()} Loss </b>',
            'yaxis': {
                'title': "<b> Loss </b>"
            },
            'xaxis': {
                'title': "<b> Epoch </b>"
            }
        }
    }
    return json.dumps(loss_graph, cls=plotly.utils.PlotlyJSONEncoder)


def get_loss_graph_log(mode):
    global performance
    loss_graph = {
        'data': [Scatter(x=performance[key][mode]['index'],
                         y=np.log10(performance[key][mode]['values']),
                         name=key) for key in performance.keys()],
        'layout': {
            'title': f'<b> {mode.capitalize()} Log Loss </b>',
            'yaxis': {
                'title': "<b> Log Loss </b>"
            },
            'xaxis': {
                'title': "<b> Epoch </b>"
            }
        }
    }
    return json.dumps(loss_graph, cls=plotly.utils.PlotlyJSONEncoder)


def get_eval_table(mode):
    global performance
    global idx
    keys = ['Name', 'Accuracy', 'Classification Error', 'Precision', 'Recall', 'Specificity', 'F1-Score', 'TP', 'FP',
            'TN', 'FN']
    table = Figure([Table(
        header=dict(
            values=keys,
            font=dict(size=12),
            align="left"
        ),
        cells=dict(
            values=get_evaluation_metrics(performance, mode, idx),
            align="left")
    )
    ])
    return json.dumps(table, cls=plotly.utils.PlotlyJSONEncoder)


# TODO Verify this function works
def get_evaluation_metrics(performance, mode, idx=0):
    return [

        [key for key in performance.keys() if
         len(performance[key]['evaluation'][mode].keys()) > 0],

        [performance[key]['evaluation'][mode]['Accuracy'][idx] for key in performance.keys() if
         len(performance[key]['evaluation'][mode].keys()) > 0],

        [performance[key]['evaluation'][mode]['Classification Error'][idx] for key in performance.keys() if
         len(performance[key]['evaluation'][mode].keys()) > 0],

        [performance[key]['evaluation'][mode]['Precision'][idx] for key in performance.keys() if
         len(performance[key]['evaluation'][mode].keys()) > 0],

        [performance[key]['evaluation'][mode]['Recall'][idx] for key in performance.keys() if
         len(performance[key]['evaluation'][mode].keys()) > 0],

        [performance[key]['evaluation'][mode]['Specificity'][idx] for key in performance.keys() if
         len(performance[key]['evaluation'][mode].keys()) > 0],

        [performance[key]['evaluation'][mode]['F1-Score'][idx] for key in performance.keys() if
         len(performance[key]['evaluation'][mode].keys()) > 0],

        [performance[key]['evaluation'][mode]['TP'][idx] for key in performance.keys() if
         len(performance[key]['evaluation'][mode].keys()) > 0],

        [performance[key]['evaluation'][mode]['FP'][idx] for key in performance.keys() if
         len(performance[key]['evaluation'][mode].keys()) > 0],

        [performance[key]['evaluation'][mode]['TN'][idx] for key in performance.keys() if
         len(performance[key]['evaluation'][mode].keys()) > 0],

        [performance[key]['evaluation'][mode]['FN'][idx] for key in performance.keys() if
         len(performance[key]['evaluation'][mode].keys()) > 0],
    ]


def add_model(performance_dict, name):
    performance_dict[name] = {'training': {'values': [], 'index': []},
                              'validation': {'values': [], 'index': []},
                              'evaluation': {'training': dict(), 'validation': dict()},
                              'model_details': dict()}
    return performance_dict


def start(ip='0.0.0.0', port=5000):
    app.run(host=ip, port=port)
